@@ -205,9 +205,7 @@ caja_launch_application_by_uri (GAppInfo *application,
     CajaFile    *file;
     gboolean        result;
     GError *error;
-#if GTK_CHECK_VERSION (3, 0, 0)
     GdkDisplay *display;
-#endif
     GdkAppLaunchContext *launch_context;
     CajaIconInfo *icon;
     int count, total;
@@ -231,7 +229,6 @@ caja_launch_application_by_uri (GAppInfo *application,
     }
     locations = g_list_reverse (locations);
 
-#if GTK_CHECK_VERSION (3, 0, 0)
     if (parent_window != NULL) {
             display = gtk_widget_get_display (GTK_WIDGET (parent_window));
     } else {
@@ -244,15 +241,11 @@ caja_launch_application_by_uri (GAppInfo *application,
         gdk_app_launch_context_set_screen (launch_context,
                                            gtk_window_get_screen (parent_window));
     }
-#else
-    launch_context = gdk_app_launch_context_new ();
-    if (parent_window)
-        gdk_app_launch_context_set_screen (launch_context,
-                                           gtk_window_get_screen (parent_window));
-#endif
 
     file = caja_file_get_by_uri (uris->data);
-    icon = caja_file_get_icon (file, 48, 0);
+    icon = caja_file_get_icon (file,
+                               48, gtk_widget_get_scale_factor (GTK_WIDGET (parent_window)),
+                               0);
     caja_file_unref (file);
     if (icon)
     {
@@ -354,7 +347,6 @@ caja_launch_application_from_command (GdkScreen  *screen,
     }
     else
     {
-#if GTK_CHECK_VERSION (3, 0, 0)
         GdkAppLaunchContext *launch_context;
         GdkDisplay *display;
         GAppInfo *app_info = NULL;
@@ -371,38 +363,6 @@ caja_launch_application_from_command (GdkScreen  *screen,
             g_object_unref (launch_context);
             g_object_unref (app_info);
         }
-#else
-        GError *error = NULL;
-        gchar **argv = NULL;
-        char* display;
-        GPid  pid;
-
-        if (!g_shell_parse_argv (full_command, NULL, &argv, &error)) {
-            g_error_free (error);
-            g_free (full_command);
-            return;
-        }
-
-        display = gdk_screen_make_display_name (screen);
-
-        g_spawn_async (
-            NULL, /* working directory */
-            argv,
-            NULL, /* envp */
-            G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD,
-            set_environment,
-            display,
-            &pid,
-            &error);
-
-        if (error != NULL) {
-            g_error_free (error);
-        } else {
-            g_child_watch_add(pid, dummy_child_watch, NULL);
-        }
- 
-        g_free(display);
-#endif
     }
 
     g_free (full_command);
@@ -450,7 +410,6 @@ caja_launch_application_from_command_array (GdkScreen  *screen,
     }
     else
     {
-#if GTK_CHECK_VERSION (3, 0, 0)
         GdkAppLaunchContext *launch_context;
         GdkDisplay *display;
         GAppInfo *app_info = NULL;
@@ -467,38 +426,6 @@ caja_launch_application_from_command_array (GdkScreen  *screen,
             g_object_unref (launch_context);
             g_object_unref (app_info);
         }
-#else
-        GError *error = NULL;
-        gchar **argv = NULL;
-        char* display;
-        GPid  pid;
-
-        if (!g_shell_parse_argv (full_command, NULL, &argv, &error)) {
-            g_error_free (error);
-            g_free (full_command);
-            return;
-        }
-
-        display = gdk_screen_make_display_name (screen);
-
-        g_spawn_async (
-            NULL, /* working directory */
-            argv,
-            NULL, /* envp */
-            G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD,
-            set_environment,
-            display,
-            &pid,
-            &error);
-
-        if (error != NULL) {
-            g_error_free (error);
-        } else {
-            g_child_watch_add(pid, dummy_child_watch, NULL);
-        }
- 
-        g_free(display);
-#endif
     }
 
     g_free (full_command);
@@ -594,11 +521,9 @@ caja_launch_desktop_file (GdkScreen   *screen,
     }
 
     error = NULL;
-#if GTK_CHECK_VERSION (3, 0, 0)
+
     context = gdk_display_get_app_launch_context (gtk_widget_get_display (GTK_WIDGET (parent_window)));
-#else
-    context = gdk_app_launch_context_new ();
-#endif
+
     /* TODO: Ideally we should accept a timestamp here instead of using GDK_CURRENT_TIME */
     gdk_app_launch_context_set_timestamp (context, GDK_CURRENT_TIME);
     gdk_app_launch_context_set_screen (context,
