@@ -27,9 +27,11 @@
 #ifndef CAJA_APPLICATION_H
 #define CAJA_APPLICATION_H
 
+#include <config.h>
 #include <gdk/gdk.h>
 #include <gio/gio.h>
-#include <unique/unique.h>
+#include <gtk/gtk.h>
+
 #include <libegg/eggsmclient.h>
 
 #define CAJA_DESKTOP_ICON_VIEW_IID "OAFIID:Caja_File_Manager_Desktop_Icon_View"
@@ -57,12 +59,13 @@ typedef struct CajaWindow CajaWindow;
 typedef struct _CajaSpatialWindow CajaSpatialWindow;
 #endif
 
-typedef struct CajaShell CajaShell;
+typedef struct _CajaApplicationPriv CajaApplicationPriv;
 
 typedef struct
 {
-    GObject parent;
-    UniqueApp* unique_app;
+    GtkApplication parent;
+    CajaApplicationPriv *priv;
+
     EggSMClient* smclient;
     GVolumeMonitor* volume_monitor;
     unsigned int automount_idle_id;
@@ -74,21 +77,12 @@ typedef struct
 
 typedef struct
 {
-    GObjectClass parent_class;
+	GtkApplicationClass parent_class;
 } CajaApplicationClass;
 
-GType                caja_application_get_type          (void);
-CajaApplication *caja_application_new               (void);
-void                 caja_application_startup           (CajaApplication *application,
-        gboolean             kill_shell,
-        gboolean             no_default_window,
-        gboolean             no_desktop,
-        gboolean             browser_window,
-        const char          *default_geometry,
-        char               **urls);
-GList *              caja_application_get_window_list           (void);
-GList *              caja_application_get_spatial_window_list    (void);
-unsigned int         caja_application_get_n_windows            (void);
+GType caja_application_get_type (void);
+
+CajaApplication *caja_application_new (void);
 
 CajaWindow *     caja_application_get_spatial_window     (CajaApplication *application,
         CajaWindow      *requesting_window,
@@ -98,15 +92,11 @@ CajaWindow *     caja_application_get_spatial_window     (CajaApplication *appli
         gboolean        *existing);
 
 CajaWindow *     caja_application_create_navigation_window     (CajaApplication *application,
-        const char          *startup_id,
         GdkScreen           *screen);
-
-void caja_application_close_all_navigation_windows (void);
+void caja_application_close_all_navigation_windows (CajaApplication *self);
 void caja_application_close_parent_windows     (CajaSpatialWindow *window);
 void caja_application_close_all_spatial_windows  (void);
-void caja_application_open_desktop      (CajaApplication *application);
-void caja_application_close_desktop     (void);
-gboolean caja_application_save_accel_map    (gpointer data);
+
 void caja_application_open_location (CajaApplication *application,
         GFile *location,
         GFile *selection,
