@@ -739,7 +739,7 @@ make_drag_image (CajaPropertyBrowser *property_browser, const char* file_name)
         else
         {
             icon_name = caja_emblem_get_icon_name_from_keyword (file_name);
-            info = caja_icon_info_lookup_from_name (icon_name, CAJA_ICON_SIZE_STANDARD);
+            info = caja_icon_info_lookup_from_name (icon_name, CAJA_ICON_SIZE_STANDARD, 1);
             pixbuf = caja_icon_info_get_pixbuf_at_size (info, CAJA_ICON_SIZE_STANDARD);
             g_object_unref (info);
             g_free (icon_name);
@@ -1830,7 +1830,7 @@ make_properties_from_directories (CajaPropertyBrowser *property_browser)
                 g_free (object_name);
                 continue;
             }
-            info = caja_icon_info_lookup_from_name (icon_name, CAJA_ICON_SIZE_STANDARD);
+            info = caja_icon_info_lookup_from_name (icon_name, CAJA_ICON_SIZE_STANDARD, 1);
             object_pixbuf = caja_icon_info_get_pixbuf_at_size (info, CAJA_ICON_SIZE_STANDARD);
             object_label = g_strdup (caja_icon_info_get_display_name (info));
             g_object_unref (info);
